@@ -56,7 +56,6 @@
 
 #include "pos.h"
 #include "osm.h"
-#include "potlatch_conf.h"
 
 #include "canvas.h"
 
@@ -79,7 +78,6 @@ typedef struct appdata_s {
   GtkWidget *vbox;
   struct map_s *map;
   osm_t *osm;
-  potlatch_conf_t *potlatch;
 
   struct statusbar_s *statusbar;
   struct settings_s *settings;
@@ -139,5 +137,7 @@ typedef struct appdata_s {
 #include "relation_edit.h"
 #include "misc.h"
 #include "map_edit.h"
+#include "josm_elemstyles.h"
+#include "style.h"
 
 #endif // APPDATA_H
