@@ -62,11 +62,12 @@ static presets_value_t *xmlGetPropValues(xmlNode *node, char *prop) {
   return value;
 }
 
-static char *icon_name_adjust(char *name) {
+char *josm_icon_name_adjust(char *name) {
   /* the icon loader uses names without extension */
   if(!strcasecmp(name+strlen(name)-4, ".png"))
     name[strlen(name)-4] = 0;
 
+#ifdef JOSM_PATH_ADJUST
   if(strncmp(name, "presets/", strlen("presets/")) != 0) {
     if(strrchr(name, '/')) {
       char *new = g_strdup_printf("presets%s", strrchr(name, '/'));
@@ -76,6 +77,7 @@ static char *icon_name_adjust(char *name) {
       printf("icon path adjusted to %s\n", name);
     }
   }
+#endif
   return name;
 }
 
@@ -89,7 +91,7 @@ static presets_item_t *parse_item(xmlDocPtr doc, xmlNode *a_node) {
   item->name = (char*)xmlGetProp(a_node, BAD_CAST "name");
 
   if((item->icon = (char*)xmlGetProp(a_node, BAD_CAST "icon"))) 
-    item->icon = icon_name_adjust(item->icon);
+    item->icon = josm_icon_name_adjust(item->icon);
 
   presets_widget_t **widget = &item->widget;
 
@@ -130,8 +132,10 @@ static presets_item_t *parse_item(xmlDocPtr doc, xmlNode *a_node) {
 	(*widget)->type = WIDGET_TYPE_COMBO;
 	(*widget)->text = (char*)xmlGetProp(cur_node, BAD_CAST "text");
 	(*widget)->key = (char*)xmlGetProp(cur_node, BAD_CAST "key");
-	(*widget)->del_if_empty = xmlGetPropIs(cur_node, "delete_if_empty", "true");
-	(*widget)->combo_w.def = (char*)xmlGetProp(cur_node, BAD_CAST "default");
+	(*widget)->del_if_empty = xmlGetPropIs(cur_node, 
+					       "delete_if_empty", "true");
+	(*widget)->combo_w.def = (char*)xmlGetProp(cur_node, 
+						   BAD_CAST "default");
 	(*widget)->combo_w.values = xmlGetPropValues(cur_node, "values");
 	widget = &((*widget)->next);
 
@@ -151,7 +155,8 @@ static presets_item_t *parse_item(xmlDocPtr doc, xmlNode *a_node) {
 	(*widget)->type = WIDGET_TYPE_CHECK;
 	(*widget)->text = (char*)xmlGetProp(cur_node, BAD_CAST "text");
 	(*widget)->key = (char*)xmlGetProp(cur_node, BAD_CAST "key");
-	(*widget)->del_if_empty = xmlGetPropIs(cur_node, "delete_if_empty", "true");
+	(*widget)->del_if_empty = xmlGetPropIs(cur_node, 
+					       "delete_if_empty", "true");
 	(*widget)->check_w.def = xmlGetPropIs(cur_node, "default", "on");
 	widget = &((*widget)->next);
 
@@ -172,7 +177,7 @@ static presets_item_t *parse_group(xmlDocPtr doc, xmlNode *a_node) {
   group->name = (char*)xmlGetProp(a_node, BAD_CAST "name");
 
   if((group->icon = (char*)xmlGetProp(a_node, BAD_CAST "icon"))) 
-    group->icon = icon_name_adjust(group->icon);
+    group->icon = josm_icon_name_adjust(group->icon);
 
   presets_item_t **preset = &group->group;
 
@@ -239,20 +244,12 @@ static presets_item_t *parse_doc(xmlDocPtr doc) {
 presets_item_t *josm_presets_load(void) {
   presets_item_t *presets = NULL;
 
-  printf("Loading JOSM presets\n");
+  printf("Loading JOSM presets ...\n");
     
   LIBXML_TEST_VERSION;
 
-  char *filename = g_strdup("data/presets.xml");
-  if(!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
-    g_free(filename);
-    filename = g_strdup("/usr/share/" PACKAGE "/presets.xml");
-    if(!g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
-      printf("JOSM presets file not present!\n");
-      g_free(filename);
-      return NULL;
-    }
-  }
+  char *filename = find_file("presets.xml");
+  if(!filename) return NULL;
   
   /* parse the file and get the DOM */
   xmlDoc *doc = NULL;
@@ -594,7 +591,8 @@ cb_menu_item(GtkMenuItem *menu_item, gpointer data) {
   }
 }
 
-static GtkWidget *build_menu(presets_context_t *context, presets_item_t *item) {
+static GtkWidget *build_menu(presets_context_t *context, 
+			     presets_item_t *item) {
   GtkWidget *menu = gtk_menu_new();
   
   while(item) {
@@ -606,7 +604,7 @@ static GtkWidget *build_menu(presets_context_t *context, presets_item_t *item) {
       else {
 	menu_item = gtk_image_menu_item_new_with_label(item->name);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item), 
-			      icon_widget_load(context->appdata, item->icon));
+		      icon_widget_load(&context->appdata->icon, item->icon));
       }
 
       if(item->is_group) 
@@ -614,7 +612,8 @@ static GtkWidget *build_menu(presets_context_t *context, presets_item_t *item) {
 				  build_menu(context, item->group));
       else {
 	g_object_set_data(G_OBJECT(menu_item), "item", item);
-	g_signal_connect(menu_item, "activate", GTK_SIGNAL_FUNC(cb_menu_item), context);
+	g_signal_connect(menu_item, "activate", 
+			 GTK_SIGNAL_FUNC(cb_menu_item), context);
       }
     } else
       menu_item = gtk_separator_menu_item_new();
@@ -739,6 +738,6 @@ static void free_items(presets_item_t *item) {
   }
 }
 
-void josm_presets_clear(presets_item_t *presets) {
+void josm_presets_free(presets_item_t *presets) {
   free_items(presets);
 }
