@@ -23,57 +23,6 @@
 
 #undef DESTROY_WAIT_FOR_GTK
 
-void map_way_color(potlatch_conf_t *potlatch, way_t *way) {
-  way->color = way->fill_color = MAP_COLOR_NONE;
-  way->draw_flags = 0;
-
-  potlatch_color_t *plcol = potlatch->colors;
-
-  /* a highway/waterway/railway is never a polygon */
-  if(!osm_way_is_way(way)) {
-  
-    /* check if this way is a polygon */
-    if(way->node_chain && 
-       (osm_way_get_last_node(way) == way->node_chain->node)) {
-      way->draw_flags |= OSM_DRAW_FLAG_POLYGON;
-      
-      /* default fill color is light grey */
-      way->fill_color = DEF_FILL_COLOR;
-    }
-  }
-  
-  while(plcol) {
-    if(osm_way_has_key_or_value(way, plcol->tag)) {
-
-      if(plcol->color != POTLATCH_COLOR_NONE) 
-	way->color = (plcol->color << 8) | 0xff;
-
-      if(plcol->stroke) 
-	way->draw_flags |= OSM_DRAW_FLAG_STROKE;
-      
-      if(plcol->fill_color != POTLATCH_COLOR_NONE)
-	way->fill_color = (plcol->fill_color << 8) | FILL_TRANSP;
-    }
-    plcol = plcol->next;
-  }
-  
-  /* use dark grey/no stroke/not filled for everything unknown */
-  if(way->color == MAP_COLOR_NONE)
-    way->color = DARK_GREY;
-}
-
-static void map_init_colors(potlatch_conf_t *potlatch, osm_t *osm) {
-  way_t *way = osm->way;
-
-  printf("preparing colors\n");
-
-  /* colorize ways */
-  while(way) {
-    map_way_color(potlatch, way);
-    way = way->next;
-  }
-}
-
 static void map_statusbar(map_t *map, map_item_t *map_item) {
   char *item_str = NULL;
   item_id_t id = ID_ILLEGAL;
@@ -193,8 +142,14 @@ static void map_node_select(appdata_t *appdata, node_t *node) {
   map_item_t *new_map_item = g_new0(map_item_t, 1);
   memcpy(new_map_item, map_item, sizeof(map_item_t));
   new_map_item->highlight = TRUE;
-  map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
-		    x, y, 2*RADIUS, HIGHLIGHT_COLOR);
+
+  float diameter = map->style->highlight.width + map->style->node.radius;
+  if(!node->ways) diameter += map->style->node.border_radius;
+  if(node->icon_buf && map->style->icon.enable) 
+    diameter = 16 * map->style->icon.scale;  // icon width = 24
+
+    map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
+		      x, y, diameter, map->style->highlight.color);
 
   if(!map_item->item) {
     /* and draw a fake node */
@@ -202,7 +157,8 @@ static void map_node_select(appdata_t *appdata, node_t *node) {
     memcpy(new_map_item, map_item, sizeof(map_item_t));
     new_map_item->highlight = TRUE;
     map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
-		      x, y, RADIUS, HIGHLIGHT2_COLOR);
+		      x, y, map->style->node.radius, 
+		      map->style->highlight.node_color);
   }
 }
 
@@ -221,12 +177,52 @@ void map_way_select(appdata_t *appdata, way_t *way) {
   icon_bar_map_item_selected(appdata, map_item, TRUE);
   gtk_widget_set_sensitive(appdata->menu_item_map_hide_sel, TRUE);
 
+  gint arrow_width = (map_item->way->draw.flags & OSM_DRAW_FLAG_BG)?
+    map->style->highlight.width + map_item->way->draw.bg.width/2:
+    map->style->highlight.width + map_item->way->draw.width/2;
+  
   node_chain_t *node_chain = map_item->way->node_chain;
+  node_t *last = NULL;
   while(node_chain) {
     map_item_t item;
     item.type = MAP_TYPE_NODE;
     item.node = node_chain->node;
 
+    /* draw an arrow between every two nodes */
+    if(last) {
+      /* create a new map item for every arrow */
+      map_item_t *new_map_item = g_new0(map_item_t, 1);
+      new_map_item->type = MAP_TYPE_WAY;
+      new_map_item->way = way;
+      new_map_item->highlight = TRUE;
+
+      struct { float x, y;} center, diff;
+      center.x = (last->lpos.x + node_chain->node->lpos.x)/2;
+      center.y = (last->lpos.y + node_chain->node->lpos.y)/2;
+      diff.x = node_chain->node->lpos.x - last->lpos.x;
+      diff.y = node_chain->node->lpos.y - last->lpos.y;
+
+      float len = sqrt(pow(diff.x, 2)+pow(diff.y, 2));
+      if(len > map->style->highlight.arrow_limit*arrow_width) {
+	len /= arrow_width;
+	diff.x = diff.x / len; 
+	diff.y = diff.y / len; 
+
+	canvas_points_t *points = canvas_points_new(4);
+	points->coords[2*0+0] = points->coords[2*3+0] = center.x + diff.x;
+	points->coords[2*0+1] = points->coords[2*3+1] = center.y + diff.y;
+	points->coords[2*1+0] = center.x + diff.y - diff.x;
+	points->coords[2*1+1] = center.y - diff.x - diff.y;
+	points->coords[2*2+0] = center.x - diff.y - diff.x;
+	points->coords[2*2+1] = center.y + diff.x - diff.y;
+	
+	map_hl_polygon_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
+			   points, map->style->highlight.arrow_color);
+	
+	canvas_points_free(points);
+      }
+    }
+      
     if(!map_hl_item_is_highlighted(map, &item)) {
 
       /* create a new map item for every node */
@@ -239,9 +235,11 @@ void map_way_select(appdata_t *appdata, way_t *way) {
       gint y = node_chain->node->lpos.y;
 
       map_hl_circle_new(map, CANVAS_GROUP_NODES_HL, new_map_item, 
-			x, y, RADIUS, HIGHLIGHT2_COLOR);
+			x, y, map->style->node.radius, 
+			map->style->highlight.node_color);
     }
 
+    last = node_chain->node;
     node_chain = node_chain->next;
   }
 
@@ -265,8 +263,10 @@ void map_way_select(appdata_t *appdata, way_t *way) {
     new_map_item->highlight = TRUE;
     
     map_hl_polyline_new(map, CANVAS_GROUP_WAYS_HL, new_map_item, points, 
-			(map_item->way->draw_flags & OSM_DRAW_FLAG_STROKE)?
-	                 2*STROKE_WIDTH:3*WAY_WIDTH, HIGHLIGHT_COLOR);
+		(map_item->way->draw.flags & OSM_DRAW_FLAG_BG)?
+		2*map->style->highlight.width + map_item->way->draw.bg.width:
+		2*map->style->highlight.width + map_item->way->draw.width, 
+		map->style->highlight.color);
 
     canvas_points_free(points);
   }
@@ -359,9 +359,19 @@ static canvas_item_t *map_node_new(map_t *map, node_t *node, gint radius,
   map_item_t *map_item = g_new0(map_item_t, 1);
   map_item->type = MAP_TYPE_NODE;
   map_item->node = node;
-  map_item->item = canvas_circle_new(map, CANVAS_GROUP_NODES, 
-		     node->lpos.x, node->lpos.y, radius, width, fill, border);
-  
+
+  if(!node->icon_buf || !map->style->icon.enable)
+    map_item->item = canvas_circle_new(map, CANVAS_GROUP_NODES, 
+       node->lpos.x, node->lpos.y, radius, width, fill, border);
+  else
+    map_item->item = canvas_image_new(map, CANVAS_GROUP_NODES, 
+      node->icon_buf, 
+      node->lpos.x - map->style->icon.scale * 
+		      gdk_pixbuf_get_width(node->icon_buf)/2, 
+      node->lpos.y - map->style->icon.scale * 
+		      gdk_pixbuf_get_height(node->icon_buf)/2, 
+	      map->style->icon.scale,map->style->icon.scale);
+
   /* attach map_item to nodes map_item_chain */
   map_item_chain_t **chain = &node->map_item_chain;
   while(*chain) chain = &(*chain)->next;
@@ -402,16 +412,20 @@ static canvas_item_t *map_way_single_new(map_t *map, way_t *way, gint radius,
   return map_item->item;
 }
 
-static canvas_item_t *map_way_new(map_t *map, canvas_group_t group, way_t *way, 
-		  canvas_points_t *points, gint width, canvas_color_t color,
-		  canvas_color_t fill_color) {
+static canvas_item_t *map_way_new(map_t *map, canvas_group_t group, 
+	  way_t *way, canvas_points_t *points, gint width, 
+	  canvas_color_t color, canvas_color_t fill_color) {
   map_item_t *map_item = g_new0(map_item_t, 1);
   map_item->type = MAP_TYPE_WAY;
   map_item->way = way;
 
-  if(way->draw_flags & OSM_DRAW_FLAG_POLYGON) {
-    map_item->item = canvas_polygon_new(map, group, points, 
-					width, color, fill_color);
+  if(way->draw.flags & OSM_DRAW_FLAG_AREA) {
+    if(map->style->area.opaque)
+      map_item->item = canvas_polygon_new(map, group, points, 
+					  width, color, fill_color);
+    else
+      map_item->item = canvas_polyline_new(map, group, points, 
+					   width, color);
   } else {
     map_item->item = canvas_polyline_new(map, group, points, width, color);
   }
@@ -431,7 +445,8 @@ static canvas_item_t *map_way_new(map_t *map, canvas_group_t group, way_t *way,
 }
 
 void map_show_node(map_t *map, node_t *node) {
-  map_node_new(map, node, S_RADIUS, 0, NODE_WAY_COLOR, 0);
+  map_node_new(map, node, map->style->node.radius, 0,
+	       RGB2CANVAS(map->style->node.color), 0);
 }
 
 void map_way_draw(map_t *map, way_t *way) {
@@ -445,7 +460,8 @@ void map_way_draw(map_t *map, way_t *way) {
   guint nodes = osm_way_number_of_nodes(way);
   if(nodes == 1) {
     /* draw a single dot where this single node is */
-    map_way_single_new(map, way, S_RADIUS, 0, NODE_WAY_COLOR, 0);
+    map_way_single_new(map, way, map->style->node.radius, 0, 
+		       RGB2CANVAS(map->style->node.color), 0);
   } else {
     canvas_points_t *points = canvas_points_new(nodes);
     
@@ -457,16 +473,16 @@ void map_way_draw(map_t *map, way_t *way) {
     }
     
     /* draw way */
-    if(way->draw_flags & OSM_DRAW_FLAG_POLYGON) {
+    if(way->draw.flags & OSM_DRAW_FLAG_AREA) {
       map_way_new(map, CANVAS_GROUP_POLYGONS, way, points, 
-		  2, DARK_GREY, way->fill_color);
+		  way->draw.width, way->draw.color, way->draw.area.color);
     } else {
       map_way_new(map, CANVAS_GROUP_WAYS, way, points, 
-		  WAY_WIDTH, way->color, NO_COLOR);
+		  way->draw.width, way->draw.color, NO_COLOR);
       
-      if(way->draw_flags & OSM_DRAW_FLAG_STROKE)
+      if(way->draw.flags & OSM_DRAW_FLAG_BG)
 	map_way_new(map, CANVAS_GROUP_WAYS_OL, way, points, 
-		    STROKE_WIDTH, DARK_GREY, NO_COLOR);
+		    way->draw.bg.width, way->draw.bg.color, NO_COLOR);
     }
     canvas_points_free(points);
   }
@@ -476,11 +492,19 @@ void map_node_draw(map_t *map, node_t *node) {
   /* don't draw a node that's not there anymore */
   if(node->flags & OSM_FLAG_DELETED)
     return;
-  
+
   if(!node->ways) 
-    map_node_new(map, node, RADIUS, RADIUS/2.0, NODE_COLOR, NODE_WAY_COLOR);
-  else if(osm_node_has_tag(node)) 
-    map_node_new(map, node, S_RADIUS, 0, NODE_WAY_COLOR, 0);
+    map_node_new(map, node, 
+		 map->style->node.radius,
+		 map->style->node.border_radius,
+		 RGBA2CANVAS(map->style->node.fill_color, 
+			     map->style->node.has_fill_color?0xff:0x00), 
+		 RGB2CANVAS(map->style->node.color));
+  
+  else if(map->style->node.show_untagged || osm_node_has_tag(node)) 
+    map_node_new(map, node, 
+		 map->style->node.radius, 0,
+		 RGB2CANVAS(map->style->node.color), 0);
 }
 
 static void map_item_draw(map_t *map, map_item_t *map_item) {
@@ -515,9 +539,9 @@ static void map_item_remove(map_t *map, map_item_t *map_item) {
   map_item_chain_destroy(chainP);
 }
 
-static void map_item_init(potlatch_conf_t *potlatch, map_item_t *map_item) {
+static void map_item_init(style_t *style, map_item_t *map_item) {
   if(map_item->type == MAP_TYPE_WAY)
-    map_way_color(potlatch, map_item->way);
+    josm_elemstyles_colorize_way(style, map_item->way);
 }
 
 void map_item_redraw(appdata_t *appdata, map_item_t *map_item) {
@@ -531,7 +555,7 @@ void map_item_redraw(appdata_t *appdata, map_item_t *map_item) {
   }
 
   map_item_remove(appdata->map, &item);
-  map_item_init(appdata->potlatch, &item);
+  map_item_init(appdata->map->style, &item);
   map_item_draw(appdata->map, &item);
 
   /* restore selection if there was one */
@@ -539,67 +563,61 @@ void map_item_redraw(appdata_t *appdata, map_item_t *map_item) {
     map_item_select(appdata, &item);
 }
 
+static void map_frisket_rectangle(canvas_points_t *points, 
+				  gint x0, gint x1, gint y0, gint y1) {
+  points->coords[2*0+0] = points->coords[2*3+0] = points->coords[2*4+0] = x0;
+  points->coords[2*1+0] = points->coords[2*2+0] = x1;
+  points->coords[2*0+1] = points->coords[2*1+1] = points->coords[2*4+1] = y0;
+  points->coords[2*2+1] = points->coords[2*3+1] = y1;
+}
+
 /* Draw the frisket area which masks off areas it'd be unsafe to edit,
  * plus its inner edge marker line */
 void map_frisket_draw(map_t *map, bounds_t *bounds) {
+  canvas_points_t *points = canvas_points_new(5);
 
-    // Edge marker line
-    gint ew2 = FRISKET_EDGE_WIDTH/2;
-    lpos_t i2a; i2a.x = bounds->min.x-ew2; i2a.y = bounds->min.y-ew2;
-    lpos_t i2b; i2b.x = bounds->max.x+ew2; i2b.y = bounds->min.y-ew2;
-    lpos_t i2c; i2c.x = bounds->max.x+ew2; i2c.y = bounds->max.y+ew2;
-    lpos_t i2d; i2d.x = bounds->min.x-ew2; i2d.y = bounds->max.y+ew2;
+  /* don't draw frisket at all if it's completely transparent */
+  if(map->style->frisket.opaque) {
+    elemstyle_color_t color = 
+      (map->style->background.color<<8) | map->style->frisket.opaque;
 
-    // Box inner window
-    lpos_t ia; ia.x = bounds->min.x; ia.y = bounds->min.y;
-    lpos_t ib; ib.x = bounds->max.x; ib.y = bounds->min.y;
-    lpos_t ic; ic.x = bounds->max.x; ic.y = bounds->max.y;
-    lpos_t id; id.x = bounds->min.x; id.y = bounds->max.y;
+    float mult = map->style->frisket.mult;
 
-    // Box outer edge
-    lpos_t oa; oa.x = FRISKET_MULT*ia.x; oa.y = FRISKET_MULT*ia.y;
-    lpos_t ob; ob.x = FRISKET_MULT*ib.x; ob.y = FRISKET_MULT*ib.y;
-    lpos_t oc; oc.x = FRISKET_MULT*ic.x; oc.y = FRISKET_MULT*ic.y;
-    lpos_t od; od.x = FRISKET_MULT*id.x; od.y = FRISKET_MULT*id.y;
-
-    canvas_points_t *points;
-
-    // Outer frisket matte
-    points = canvas_points_new(7);
-    canvas_point_set_pos(points, 0, &oa);
-    canvas_point_set_pos(points, 1, &ob);
-    canvas_point_set_pos(points, 2, &oc);
-    canvas_point_set_pos(points, 3, &ic);
-    canvas_point_set_pos(points, 4, &ib);
-    canvas_point_set_pos(points, 5, &ia);
-    canvas_point_set_pos(points, 6, &oa);
-    canvas_polygon_new(map, CANVAS_GROUP_NODES, points,
-                       1, NO_COLOR, FRISKET_COLOR);
-    canvas_points_free(points);
-
-    points = canvas_points_new(7);
-    canvas_point_set_pos(points, 0, &oc);
-    canvas_point_set_pos(points, 1, &od);
-    canvas_point_set_pos(points, 2, &oa);
-    canvas_point_set_pos(points, 3, &ia);
-    canvas_point_set_pos(points, 4, &id);
-    canvas_point_set_pos(points, 5, &ic);
-    canvas_point_set_pos(points, 6, &oc);
-    canvas_polygon_new(map, CANVAS_GROUP_NODES, points,
-                       1, NO_COLOR, FRISKET_COLOR);
-    canvas_points_free(points);
+    /* top rectangle */
+    map_frisket_rectangle(points, mult*bounds->min.x, mult*bounds->max.x,
+			  mult*bounds->min.y, bounds->min.y);    
+    canvas_polygon_new(map, CANVAS_GROUP_NODES, points, 1, NO_COLOR, color);
     
-    // Frisket inner edge - pick out in a slightly darker colour
-    points = canvas_points_new(5);
-    canvas_point_set_pos(points, 0, &i2a);
-    canvas_point_set_pos(points, 1, &i2b);
-    canvas_point_set_pos(points, 2, &i2c);
-    canvas_point_set_pos(points, 3, &i2d);
-    canvas_point_set_pos(points, 4, &i2a);
-    canvas_polyline_new(map, CANVAS_GROUP_NODES, points,
-                        FRISKET_EDGE_WIDTH, FRISKET_EDGE_COLOR);
+    /* bottom rectangle */
+    map_frisket_rectangle(points, mult*bounds->min.x, mult*bounds->max.x,
+			  bounds->max.y, mult*bounds->max.y);
+    canvas_polygon_new(map, CANVAS_GROUP_NODES, points, 1, NO_COLOR, color);
 
-    canvas_points_free(points);
+    /* left rectangle */
+    map_frisket_rectangle(points, mult*bounds->min.x, bounds->min.x,
+			  mult*bounds->min.y, mult*bounds->max.y);
+    canvas_polygon_new(map, CANVAS_GROUP_NODES, points, 1, NO_COLOR, color);
+
+    /* right rectangle */
+    map_frisket_rectangle(points, bounds->max.x, mult*bounds->max.x,
+			  mult*bounds->min.y, mult*bounds->max.y);
+    canvas_polygon_new(map, CANVAS_GROUP_NODES, points, 1, NO_COLOR, color);
+
+  }
+
+  if(map->style->frisket.border.present) {
+    // Edge marker line
+    gint ew2 = map->style->frisket.border.width/2;
+    map_frisket_rectangle(points, 
+			  bounds->min.x-ew2, bounds->max.x+ew2,
+			  bounds->min.y-ew2, bounds->max.y+ew2);
+    
+    canvas_polyline_new(map, CANVAS_GROUP_NODES, points,
+			map->style->frisket.border.width, 
+			map->style->frisket.border.color);
+
+  }
+  canvas_points_free(points);
 }
 
 static void map_draw(map_t *map, osm_t *osm) {
@@ -672,12 +690,16 @@ static gint map_destroy_event(GtkWidget *widget, gpointer data) {
   
   map_free_map_item_chains(appdata);
 
+  /* free buffered tags */
   if(map->last_node_tags) osm_tags_free(map->last_node_tags);
   if(map->last_way_tags) osm_tags_free(map->last_way_tags);
 
   map_state_free(map->state);
-  g_free(map);
 
+  if(map->style)
+    style_free(map->style);
+
+  g_free(map);
 
   appdata->map = NULL;
 
@@ -841,6 +863,7 @@ static gboolean map_limit_zoom(map_t *map, gdouble *zoom) {
 }
 
 
+#if 0
 /* Scroll the map a little towards the centre from where it is right now.
  * This is used as a cute recentring trick when the map is at its outer
  * scroll limit. */
@@ -910,8 +933,8 @@ static void map_scroll_towards_centre(map_t *map, gdouble amt) {
     map->state->scroll_offset.x = sx;
     map->state->scroll_offset.y = sy;
 }
-
-#endif
+#endif // #if 0
+#endif // #ifdef USE_GOOCANVAS
 
 
 
@@ -1127,7 +1150,7 @@ void map_way_delete(appdata_t *appdata, way_t *way) {
 
   /* and mark it "deleted" in the database */
   osm_way_remove_from_relation(appdata->osm, way);
-  osm_way_delete(appdata->osm, way, FALSE);
+  osm_way_delete(appdata->osm, &appdata->icon, way, FALSE);
 }
 
 static void map_handle_click(appdata_t *appdata, map_t *map) {
@@ -1192,7 +1215,8 @@ static void map_touchnode_update(appdata_t *appdata, gint x, gint y) {
       gint nx = x - node->lpos.x;
       gint ny = y - node->lpos.y;
 
-      if(nx*nx + ny*ny < RADIUS*RADIUS)
+      if((nx < map->style->node.radius) && (ny < map->style->node.radius) &&
+	 (nx*nx + ny*ny < map->style->node.radius * map->style->node.radius))
 	map_hl_touchnode_draw(map, node);
     }
     node = node->next;
@@ -1206,7 +1230,8 @@ static void map_touchnode_update(appdata_t *appdata, gint x, gint y) {
       gint nx = x - chain->node->lpos.x;
       gint ny = y - chain->node->lpos.y;
 	  
-      if(nx*nx + ny*ny < RADIUS*RADIUS)
+      if((nx < map->style->node.radius) && (ny < map->style->node.radius) &&
+	 (nx*nx + ny*ny < map->style->node.radius * map->style->node.radius))
 	map_hl_touchnode_draw(map, chain->node);
 
       chain = chain->next;
@@ -1252,11 +1277,11 @@ static void map_button_press(map_t *map, gint x, gint y) {
     break;
 
   case MAP_ACTION_NODE_ADD:
-    map_hl_cursor_draw(map, x, y, FALSE, RADIUS);
+    map_hl_cursor_draw(map, x, y, FALSE, map->style->node.radius);
     break;
 
   case MAP_ACTION_WAY_ADD:
-    map_hl_cursor_draw(map, x, y, FALSE, RADIUS);
+    map_hl_cursor_draw(map, x, y, FALSE, map->style->node.radius);
     map_touchnode_update(map->appdata, x, y);
     break;
 
@@ -1453,18 +1478,18 @@ static gboolean map_motion_notify_event(GtkWidget *widget,
       if(!map->pen_down.on_selected_node)
 	map_do_scroll(map, x, y);
       else {
-	map_hl_cursor_draw(map, x, y, FALSE, RADIUS);
+	map_hl_cursor_draw(map, x, y, FALSE, map->style->node.radius);
 	map_touchnode_update(appdata, x, y);
       }
     }
     break;
     
   case MAP_ACTION_NODE_ADD:
-    map_hl_cursor_draw(map, x, y, FALSE, RADIUS);
+    map_hl_cursor_draw(map, x, y, FALSE, map->style->node.radius);
     break;
     
   case MAP_ACTION_WAY_ADD:
-    map_hl_cursor_draw(map, x, y, FALSE, RADIUS);
+    map_hl_cursor_draw(map, x, y, FALSE, map->style->node.radius);
     map_touchnode_update(appdata, x, y);
     break;
 
@@ -1557,13 +1582,14 @@ gboolean map_key_press_event(appdata_t *appdata, GdkEventKey *event) {
       break;
     }
   }
-
   return FALSE;
 }
 
 GtkWidget *map_new(appdata_t *appdata) {
   map_t *map = appdata->map = g_new0(map_t, 1);
 
+  map->style = style_load(appdata, appdata->settings->style);
+
   if(appdata->project && appdata->project->map_state) {
     printf("Using projects map state\n");
     map->state = appdata->project->map_state;
@@ -1598,6 +1624,8 @@ GtkWidget *map_new(appdata_t *appdata) {
   map->canvas = goo_canvas_new();
 
   g_object_set(G_OBJECT(map->canvas), "anchor", GTK_ANCHOR_CENTER, NULL);
+  g_object_set(G_OBJECT(map->canvas), "background-color-rgb", 
+	       map->style->background.color, NULL);
 
   GooCanvasItem *root = goo_canvas_get_root_item(GOO_CANVAS(map->canvas));
 
@@ -1632,28 +1660,19 @@ GtkWidget *map_new(appdata_t *appdata) {
 
 void map_init(appdata_t *appdata) {
   map_t *map = appdata->map;
-  gint min_x, min_y, max_x, max_y;
-  min_x = min_y = max_x = max_y = 0;
 
   /* set initial zoom */
   map_set_zoom(map, map->state->zoom, FALSE);
-  map_init_colors(appdata->potlatch, appdata->osm);
+  josm_elemstyles_colorize_world(map->style, appdata->osm);
+
   map_draw(map, appdata->osm);
 
-  min_x = appdata->osm->bounds->min.x;
-  min_y = appdata->osm->bounds->min.y;
-  max_x = appdata->osm->bounds->max.x;
-  max_y = appdata->osm->bounds->max.y;
-  min_x *= FRISKET_MULT;
-  min_y *= FRISKET_MULT;
-  max_x *= FRISKET_MULT;
-  max_y *= FRISKET_MULT;
-  
-  canvas_set_bounds(map->canvas, min_x, min_y, max_x, max_y);
-
-  //canvas_set_bounds(map->canvas,
-  //	appdata->osm->bounds->min.x, appdata->osm->bounds->min.y,
-  //	appdata->osm->bounds->max.x, appdata->osm->bounds->max.y);
+  float mult = appdata->map->style->frisket.mult;
+  canvas_set_bounds(map->canvas, 
+		    mult*appdata->osm->bounds->min.x,
+		    mult*appdata->osm->bounds->min.y,
+		    mult*appdata->osm->bounds->max.x,
+		    mult*appdata->osm->bounds->max.y);
 
   printf("restore scroll offsets %d/%d\n",
 	 map->state->scroll_offset.x, map->state->scroll_offset.y);
@@ -1682,7 +1701,7 @@ void map_item_set_flags(map_item_t *map_item, int set, int clr) {
   }
 }
 
-void map_clear(appdata_t *appdata) {
+void map_clear(appdata_t *appdata, gint layer_mask) {
   map_t *map = appdata->map;
 
   printf("freeing map contents\n");
@@ -1695,18 +1714,22 @@ void map_clear(appdata_t *appdata) {
   canvas_group_t group;
   for(group=0;group<CANVAS_GROUPS;group++) {
 
+#ifdef USE_GNOMECANVAS
     /* destroy the entire group */
     canvas_item_destroy(map->group[group]);
 
-#ifdef USE_GNOMECANVAS
     /* and create an empty new one */
     map->group[group] = gnome_canvas_item_new(
 		      gnome_canvas_root(GNOME_CANVAS(map->canvas)),
 		      GNOME_TYPE_CANVAS_GROUP,
 		      NULL);       
 #else
-    GooCanvasItem *root = goo_canvas_get_root_item(GOO_CANVAS(map->canvas));
-    map->group[group] = goo_canvas_group_new(root, NULL);       
+    if(layer_mask & (1<<group)) {
+      gint children = goo_canvas_item_get_n_children(map->group[group]);
+      printf("Removing %d children from layer %d\n", children, group);
+      while(children--) 
+	goo_canvas_item_remove_child(map->group[group], children);
+    }
 #endif
   }
 }
@@ -1714,7 +1737,7 @@ void map_clear(appdata_t *appdata) {
 void map_paint(appdata_t *appdata) {
   map_t *map = appdata->map;
 
-  map_init_colors(appdata->potlatch, appdata->osm);
+  josm_elemstyles_colorize_world(map->style, appdata->osm);
   map_draw(map, appdata->osm);
 }
 
@@ -1881,7 +1904,8 @@ void map_delete_selected(appdata_t *appdata) {
 
     /* and mark it "deleted" in the database */
     osm_node_remove_from_relation(appdata->osm, item.node);
-    way_chain_t *chain = osm_node_delete(appdata->osm, item.node, FALSE, TRUE);
+    way_chain_t *chain = osm_node_delete(appdata->osm, 
+			 &appdata->icon, item.node, FALSE, TRUE);
 
     /* redraw all affected ways */
     while(chain) {
@@ -1929,8 +1953,8 @@ void map_track_draw_seg(map_t *map, track_seg_t *seg) {
     g_assert(!seg->item);
 
     seg->item = canvas_circle_new(map, CANVAS_GROUP_TRACK,
-		  seg->track_point->lpos.x, seg->track_point->lpos.y, 
-		  1.5*WAY_WIDTH, 0, TRACK_COLOR, NO_COLOR);
+	  seg->track_point->lpos.x, seg->track_point->lpos.y, 
+	  map->style->track.width/2.0, 0, map->style->track.color, NO_COLOR);
   }
 
   if(pnum > 1) {
@@ -1951,7 +1975,7 @@ void map_track_draw_seg(map_t *map, track_seg_t *seg) {
       canvas_item_destroy(seg->item);
 
     seg->item = canvas_polyline_new(map, CANVAS_GROUP_TRACK,
-		  points, 3*WAY_WIDTH, TRACK_COLOR);
+	  points, map->style->track.width, map->style->track.color);
 
     canvas_points_free(points);
   }
@@ -2017,7 +2041,8 @@ void map_track_pos(appdata_t *appdata, lpos_t *lpos) {
 
   if(lpos)
     appdata->track.gps_item = canvas_circle_new(appdata->map, CANVAS_GROUP_GPS, 
-	lpos->x, lpos->y, 1.5 * WAY_WIDTH, 0, TRACK_GPS_COLOR, NO_COLOR);
+	lpos->x, lpos->y, appdata->map->style->track.width/2.0, 0, 
+			RGB2CANVAS(appdata->map->style->track.gps_color), NO_COLOR);
 }
 
 /* ------------------- map background ------------------ */
