@@ -277,7 +277,10 @@ gboolean osm_way_ends_with_node(way_t *way, node_t *node) {
 
 /* ------------------- node handling ------------------- */
 
-void osm_node_free(node_t *node) {
+void osm_node_free(icon_t **icon, node_t *node) {
+  if(node->icon_buf) 
+    icon_free(icon, node->icon_buf);
+
   /* there must not be anything left in this chain */
   g_assert(!node->map_item_chain);
 
@@ -285,10 +288,10 @@ void osm_node_free(node_t *node) {
   g_free(node);
 }
 
-static void osm_nodes_free(node_t *node) {
+static void osm_nodes_free(icon_t **icon, node_t *node) {
   while(node) {
     node_t *next = node->next;
-    osm_node_free(node);
+    osm_node_free(icon, node);
     node = next;
   }
 }
@@ -854,12 +857,12 @@ static osm_t *osm_parse_doc(xmlDocPtr doc) {
 
 /* ------------------ osm handling ----------------- */
 
-void osm_free(osm_t *osm) {
+void osm_free(icon_t **icon, osm_t *osm) {
   if(!osm) return;
 
   if(osm->bounds)   osm_bounds_free(osm->bounds);
   if(osm->user)     osm_users_free(osm->user);
-  if(osm->node)     osm_nodes_free(osm->node);
+  if(osm->node)     osm_nodes_free(icon, osm->node);
   if(osm->way)      osm_ways_free(osm->way);
   if(osm->relation) osm_relations_free(osm->relation);
   g_free(osm);
@@ -943,39 +946,27 @@ char *osm_node_get_value(node_t *node, char *key) {
   return NULL;
 }
 
-gboolean osm_way_has_key_or_value(way_t *way, char *str) {
+gboolean osm_way_has_value(way_t *way, char *str) {
   tag_t *tag = way->tag;
 
   while(tag) {
-    if(tag->key && strcasecmp(tag->key, str) == 0)
-      return TRUE;
-
     if(tag->value && strcasecmp(tag->value, str) == 0)
       return TRUE;
 
     tag = tag->next;
   }
-
   return FALSE;
 }
 
-gboolean osm_way_is_way(way_t *way) {
-  tag_t *tag = way->tag;
+gboolean osm_node_has_value(node_t *node, char *str) {
+  tag_t *tag = node->tag;
 
   while(tag) {
-    if(tag->key) {
-      if(strcasecmp(tag->key, "highway") == 0)
-	return TRUE;
+    if(tag->value && strcasecmp(tag->value, str) == 0)
+      return TRUE;
 
-      if(strcasecmp(tag->key, "waterway") == 0)
-	return TRUE;
-
-      if(strcasecmp(tag->key, "railway") == 0)
-	return TRUE;
-    }
     tag = tag->next;
   }
-
   return FALSE;
 }
 
@@ -1293,7 +1284,8 @@ void osm_way_attach(osm_t *osm, way_t *way) {
 }
 
 /* returns pointer to chain of ways affected by this deletion */
-way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently,
+way_chain_t *osm_node_delete(osm_t *osm, icon_t **icon, 
+			     node_t *node, gboolean permanently,
 			     gboolean affect_ways) {
   way_chain_t *way_chain = NULL, **cur_way_chain = &way_chain;
 
@@ -1350,7 +1342,7 @@ way_chain_t *osm_node_delete(osm_t *osm, node_t *node, gboolean permanently,
 	found++;
 	*cnode = (*cnode)->next;
 
-	osm_node_free(node);
+	osm_node_free(icon, node);
       } else
 	cnode = &((*cnode)->next);
     }
@@ -1539,7 +1531,8 @@ void osm_way_remove_from_relation(osm_t *osm, way_t *way) {
   }
 }
 
-void osm_way_delete(osm_t *osm, way_t *way, gboolean permanently) {
+void osm_way_delete(osm_t *osm, icon_t **icon, 
+		    way_t *way, gboolean permanently) {
 
   /* new ways aren't stored on the server and are just deleted permanently */
   if(way->flags & OSM_FLAG_NEW) {
@@ -1562,7 +1555,7 @@ void osm_way_delete(osm_t *osm, way_t *way, gboolean permanently) {
       /* associated ways as the only such way is the oen we are currently */
       /* deleting */
       way_chain_t *way_chain = 
-	osm_node_delete(osm, (*chain)->node, FALSE, FALSE);
+	osm_node_delete(osm, icon, (*chain)->node, FALSE, FALSE);
       g_assert(way_chain);
       while(way_chain) {
 	way_chain_t *way_next = way_chain->next;
